@@ -1,12 +1,13 @@
 """
-Routines to calculate the radial distribution function g(r)
-and isotropic structure factor S(q).
+Routines to calculate the radial distribution function :math:`g(r)`
+and isotropic structure factor :math:`S(q)`.
 
-See https://en.wikipedia.org/wiki/Radial_distribution_function
+See `here <https://en.wikipedia.org/wiki/Radial_distribution_function>`_
 to learn more.
 
+Adapted from https://github.com/wenyan4work/point_cloud.
+
 .. moduleauthor:: Michael O'Brien <michaelobrien@g.harvard.edu>
-.. moduleauthor:: Wen Yan
 
 """
 
@@ -18,41 +19,51 @@ from scipy.special import jv
 from time import time
 
 
-def structure_factor(gr, r, N, boxsize,
-                     qmin=None, qmax=None, dq=None, **kwargs):
+def structure_factor(gr, r, N, boxsize, q=None, **kwargs):
     """
-    Calculate the isotropic structure factor S(q) from
-    the radial distribution function g(r) of a set of particles.
+    Calculate the isotropic structure factor :math:`S(q)` from
+    the radial distribution function :math:`g(r)` of a set of :math:`N` 
+    particle positions in a 2D or 3D periodic box with volume :math:`V`.
+
+    The structure factor in 3D is computed as
+
+    .. math::
+
+        S(q) = 1 + 4\\pi \\rho \\frac{1}{q} \int dr \ r \ \\textrm{sin}(qr) [g(r) - 1]
+
+    and in 2D
+
+    .. math::
+
+        S(q) = 1 + 2\\pi \\rho \int dr \ r \ J_{0}(qr) [g(r) - 1],
+
+    where :math:`\\rho = N/V` and :math:`J_{0}` is the 0th bessel function of the first kind.
 
     Parameters
     ----------
     gr : `np.ndarray`
-        The radial distribution function outputted
-        from `spatialstats.points.point_cloud.rdf`.
+        The radial distribution function :math:`g(r)`
+        from :ref:`spatialstats.points.rdf<rdf>`.
     r : `np.ndarray`
-        The domain of the radial distribution function
-        outputted from `spatialstats.points.point_cloud.rdf`.
+        The domain of :math:`g(r)`
+        from :ref:`spatialstats.points.rdf<rdf>`.
     N : `int`
-        The number of particles.
+        The number of particles :math:`N`.
     boxsize : `list` of `float`
         The rectangular domain over which
         to apply periodic boundary conditions.
-        See `scipy.spatial.cKDTree` documentation.
-    qmin : `float`, optional
-        Minimum wavenumber for S(q). Default is ``dq``.
-    qmax : `float`, optional
-        Maximum wavenumber for S(q). Default is ``100*dq``
-    dq : `float`, optional
-        Wavenumber step size. Default is ``2*np.pi/L``, where
-        ``L = max(boxsize)``.
+        See ``scipy.spatial.cKDTree``.
+    q : `np.ndarray`, optional
+        Dimensional wavenumber bins :math:`q`.
+        If ``None``, ``q = np.arange(dq, 200*dq, dq)``
+        with ``dq = 2*np.pi / max(boxsize)``.
 
     Returns
     -------
     Sq : `np.ndarray`
-        The static structure factor.
+        The structure factor :math:`S(q)`.
     q : `np.ndarray`
-        Dimensional wavenumbers discretized by ``2*np.pi/L``, where
-        ``L = max(boxsize)``.
+        Wavenumber bins :math:`q`.
     """
     ndim = len(boxsize)
 
@@ -60,10 +71,9 @@ def structure_factor(gr, r, N, boxsize,
         raise ValueError("Dimension of space must be 2 or 3")
 
     # Generate wavenumbers
-    dq = (2*np.pi / max(boxsize)) if dq is None else dq
-    qmin = dq if qmin is None else qmin
-    qmax = 100*dq if qmax is None else qmax
-    q = np.arange(qmin, qmax+dq, dq)
+    if q is None:
+        dq = (2*np.pi / max(boxsize))
+        q = np.arange(dq, 200*dq, dq)
 
     def S(q):
         '''Integrand for isotropic structure factor'''
@@ -86,43 +96,42 @@ def structure_factor(gr, r, N, boxsize,
 
 def rdf(points, boxsize, rmin=None, rmax=None, npts=100, bench=False):
     """
-    Calculate the radial distribution function g(r)
-    for a group of particles in 2D or 3D.
+    .. _rdf:
 
-    Works for periodic boundary conditions on rectangular
-    domains.
+    Calculate the radial distribution function :math:`g(r)`
+    for a set of :math:`N` particle positions in a 2D or 3D periodic box.
 
     Parameters
     ---------
-    points : `np.ndarray`, shape `(ndim, N)`
-        Particle locations, where ndim is number
-        of dimensions and N is number of particles.
+    points : `np.ndarray`, shape `(N, ndim)`
+        Particle positions in ``ndim`` dimensions
+        and ``N`` particles. Passed to
+        ``scipy.spatial.cKDTree``.
     boxsize : `list` of `float`
         The rectangular domain over which
         to apply periodic boundary conditions.
-        See `scipy.spatial.cKDTree` documentation.
+        Passed to ``scipy.spatial.cKDTree``.
     rmin : `float`, optional
-        Minimum r value in g(r).
+        Minimum :math:`r` value in :math:`g(r)`.
     rmax : `float`, optional
         Cutoff radius for KDTree search and
-        maximum r value in g(r). Default is
-        maximum distance between any pair of
+        maximum :math:`r` value in :math:`g(r)`.
+        Default is maximum distance between any pair of
         particles.
     npts : `int`, optional
-        Number points between [`rmin`, `rmax`] in
-        g(r).
+        Number points in domain :math:`r`.
     bench : `bool`, optional
         Print message for time of calculation.
     Returns
     -------
     gr : `np.ndarray`
-        Radial distribution function g(r).
+        Radial distribution function :math:`g(r)`.
     r : `np.ndarray`
-        Radius r.
+        Radius :math:`r`.
     """
-    ndim, N = points.shape
-    points = points.T
+    N, ndim = points.shape
     rmax = min(boxsize)/2 if rmax is None else rmax
+    boxsize = np.array(boxsize)
 
     if ndim not in [2, 3]:
         raise ValueError("Dimension of space must be 2 or 3")
@@ -167,13 +176,13 @@ def _gen_rdf(rvec, npar, density, rmin, rmax, nbins):
     return r, rdf
 
 
-@nb.njit(cache=True)
+@nb.njit(parallel=True, cache=True)
 def _get_displacements(coords, pairs, boxsize, rmax):
     '''Get displacements between pairs'''
-    npairs, itpairs = len(pairs), iter(pairs)
+    npairs = pairs.shape[0]
     rvec = np.zeros((npairs, coords.shape[1]))
-    for index in range(npairs):
-        pair = next(itpairs)
+    for index in nb.prange(npairs):
+        pair = pairs[index]
         id0, id1 = pair
         pos0, pos1 = coords[id0], coords[id1]
         vec01 = pos1-pos0
@@ -188,12 +197,13 @@ def _get_displacements(coords, pairs, boxsize, rmax):
 def _get_pairs(coords, boxsize, rmax):
     '''Get coordinate pairs within distance rmax'''
     tree = ss.cKDTree(coords, boxsize=boxsize)
-    boxsize = np.array(boxsize)
-    pairs = tree.query_pairs(r=rmax)  # this returns only pairs (i<j)
-    pairs2 = set()
-    for p in pairs:
-        pairs2.add((p[1], p[0]))
-    pairs.update(pairs2)
+    # Get unique pairs (i<j)
+    temp = np.array(list(tree.query_pairs(r=rmax)), dtype=np.int)
+    # Get rest of the pairs (i>=j)
+    npairs = len(temp)
+    pairs = np.zeros(shape=(2*npairs, 2), dtype=np.int)
+    pairs[:npairs, :] = temp
+    pairs[npairs:, :] = temp[:, [1, 0]]
     return pairs
 
 
@@ -245,3 +255,28 @@ def _closest_image(target, source, boxsize):
         pos, ind = _closest_point1d(target[i], pts)
         image[i] = pos
     return image
+
+
+if __name__ == "__main__":
+
+    from matplotlib import pyplot as plt
+
+    N = 2000
+    boxsize = [100, 100]
+    data = np.random.rand(N, 2)*100
+    rmax = boxsize[0]/4
+
+    gr, r = rdf(data, boxsize, rmax=rmax, npts=200)
+
+    Sq, q = structure_factor(gr, r, N, boxsize, qmin=0, qmax=100, dq=.5)
+
+    fig, axes = plt.subplots(ncols=2)
+    axes[0].plot(r, gr)
+    axes[0].set_xlabel("$r$")
+    axes[0].set_ylabel("$g(r)$")
+
+    axes[1].plot(q, Sq)
+    axes[1].set_xlabel("$q$")
+    axes[1].set_ylabel("$S(q)$")
+
+    plt.show()
